/*
 * File   : single.c
 * Purpose: This file contains the management for manipulating singles.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xmt/InputField.h>
#include <Xmt/DialogP.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Hash.h>
#include <X11/IntrinsicP.h>
#include <X11/CompositeP.h>
#include "platten.h"

EXEC SQL include sqlca;
EXEC SQL include sqlda;
EXEC SQL include sqltypes;
EXEC SQL include datetime;

/*
 * Strukturen zum austauschen von Daten in Dialogboxen.
 */
typedef struct
{
	char songa[41];
	char songb[41];
	char singer[41];
	char date[11];
        char zeit[6];
        char zeitb[6];
} single_t;

XtResource single_resource[] = {
	{ "songA", "songA", XmtRBuffer, XmtSizeOf (single_t, songa),
	  XtOffsetOf (single_t, songa), XtRImmediate, NULL},
	{ "songB", "songB", XmtRBuffer, XmtSizeOf (single_t, songb),
	  XtOffsetOf (single_t, songb), XtRImmediate, NULL},
	{ "singer", "singer", XmtRBuffer, XmtSizeOf (single_t, singer),
	  XtOffsetOf (single_t, singer), XtRImmediate, NULL},
	{ "date", "date", XmtRBuffer, XmtSizeOf (single_t, date),
	  XtOffsetOf (single_t, date), XtRImmediate, NULL},
	{ "zeit", "zeit", XmtRBuffer, XmtSizeOf (single_t, zeit),
	  XtOffsetOf (single_t, zeit), XtRImmediate, NULL},
	{ "zeitb", "zeitb", XmtRBuffer, XmtSizeOf (single_t, zeitb),
	  XtOffsetOf (single_t, zeitb), XtRImmediate, NULL}
};

typedef struct
{
	char number[11];
	char songa[41];
	char songb[41];
	char singer[41];
	char date[11];
	char zeit[6];
	char zeitb[6];
} squery_t;

XtResource squery_resource[] = {
	{ "Nummer", "Nummer", XmtRBuffer, XmtSizeOf (squery_t, number),
	  XtOffsetOf (squery_t, number), XtRImmediate, NULL},
	{ "songA", "songA", XmtRBuffer, XmtSizeOf (squery_t, songa),
	  XtOffsetOf (squery_t, songa), XtRImmediate, NULL},
	{ "songB", "songB", XmtRBuffer, XmtSizeOf (squery_t, songb),
	  XtOffsetOf (squery_t, songb), XtRImmediate, NULL},
	{ "singer", "singer", XmtRBuffer, XmtSizeOf (squery_t, singer),
	  XtOffsetOf (squery_t, singer), XtRImmediate, NULL},
	{ "date", "date", XmtRBuffer, XmtSizeOf (squery_t, date),
	  XtOffsetOf (squery_t, date), XtRImmediate, NULL},
	{ "zeit", "zeit", XmtRBuffer, XmtSizeOf (squery_t, zeit),
	  XtOffsetOf (squery_t, zeit), XtRImmediate, NULL},
	{ "zeitb", "zeitb", XmtRBuffer, XmtSizeOf (squery_t, zeitb),
	  XtOffsetOf (squery_t, zeitb), XtRImmediate, NULL}
};

/*
 * Globale Variablen fr diese Datei.
 */
Boolean sqlQuery;
Widget QueryWidget = (Widget)0;
Widget AddWidget = (Widget)0;

/*
 * Funktionsprototypen die nur in dieser Datei verwendet werden.
 */
void AddSingle (Widget w, single_t *si);
void ActivateSearch (Boolean b);
void ActivateMove (Boolean b);
 
/*
 * Handling der Dialogbox zur Eingabe von Singles
 */
void NewSingle ()
{
single_t single;

	XmtDisplayBusyCursor (toplevel);

	if (!AddWidget)
	   AddWidget = XmtBuildQueryDialog (toplevel, "addSingle", 
		single_resource, XtNumber (single_resource), NULL);

	memset ((void *)&single, 0, sizeof (single_t));

	XmtDialogDoSync (AddWidget, &single);
	XmtDisplayDefaultCursor (toplevel);
}

void AddNewSingleCallback (Widget w, XtPointer tag, XtPointer data)
{
single_t single;

	XmtDialogGetDialogValues (AddWidget, &single);

	if (!strlen (single.songa) || !strlen (single.songb) ||
	    !strlen (single.singer))
	{
	   XmtDisplayError (w, "MissingValue", NULL);
           return;
	}

	AddSingle (AddWidget, &single);
	memset ((void *)&single, 0, sizeof (single_t));
	XmtDialogSetDialogValues (AddWidget, &single);
}

void ClearValuesCallback (Widget w, XtPointer tag, XtPointer data)
{
squery_t single;

	memset ((void *)&single, 0, sizeof (single_t));
	XmtDialogSetDialogValues (AddWidget, &single);
}

void AddSingle (Widget w, single_t *si)
{
EXEC SQL BEGIN DECLARE SECTION;
	string songa[41], songb[41], singer[41], zeit[6], zeitb[6];
	long idate;
EXEC SQL END DECLARE SECTION;
int i;
char hv0[128];

	strcpy (songa, si->songa);
	strcpy (songb, si->songb);
	strcpy (singer, si->singer);
	strcpy (zeit, si->zeit);
	strcpy (zeitb, si->zeitb);

	if ((idate = CharToIDate (w, si->date)) <= 0L)
	   return;

	XmtDisplayBusyCursor (AddWidget);

	if (has_trans)
	   EXEC SQL begin work;

	if (!strlen (si->date))
	{
	   EXEC SQL insert into disk (number, title, b_side, singer,
		pressed, zeit, zeitb)
		values (0, :songa, :songb, :singer, NULL, :zeit, :zeitb);
	}
	else
	{
	   EXEC SQL insert into disk (number, title, b_side, singer,
		pressed, zeit, zeitb)
		values (0, :songa, :songb, :singer, :idate, :zeit, :zeitb);
	}

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (AddWidget);

	   if (has_trans)
	      EXEC SQL rollback work;
	}
	else if (has_trans)
	{
	   EXEC SQL commit work;
	   XBell (XtDisplay (AddWidget), 0);
	}

	XmtDisplayDefaultCursor (AddWidget);
}

void UpdateSingleCallback (Widget w, XtPointer tag, XtPointer data)
{
EXEC SQL BEGIN DECLARE SECTION;
	string songa[41], songb[41], singer[41], zeit[6], zeitb[6];
	long idate, nummer;
EXEC SQL END DECLARE SECTION;
squery_t si;
int i;
char hv0[128];

	XmtDialogGetDialogValues (QueryWidget, &si);
	nummer = atol (si.number);
	strcpy (songa, si.songa);
	strcpy (songb, si.songb);
	strcpy (singer, si.singer);
	strcpy (zeit, si.zeit);
	strcpy (zeitb, si.zeitb);

	if (nummer <= 0 || !strlen (songa) || !strlen (songb) ||
	    !strlen (singer))
	{
	   XmtDisplayError (w, "MissingValue", NULL);
           return;
	}

	if ((idate = CharToIDate (w, si.date)) <= 0L)
	   return;

	XmtDisplayBusyCursor (QueryWidget);

	if (has_trans)
	   EXEC SQL begin work;

	if (!strlen (si.date))
	{
	   EXEC SQL update disk set title = :songa,
			b_side = :songb,
			singer = :singer,
			zeit = :zeit,
			zeitb = :zeitb
			where number = :nummer;
	}
	else
	{
	   EXEC SQL update disk set title = :songa,
			b_side = :songb,
			singer = :singer,
			pressed = :idate,
			zeit = :zeit,
			zeitb = :zeitb
			where number = :nummer;
	}

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (QueryWidget);

	   if (has_trans)
	      EXEC SQL rollback work;
	}
	else if (has_trans)
	{
	   EXEC SQL commit work;
	   XBell (XtDisplay (QueryWidget), 0);
	}

	XmtDisplayDefaultCursor (QueryWidget);
}

void SearchSingle ()
{
squery_t single;

	XmtDisplayBusyCursor (toplevel);

	if (!QueryWidget)
	   QueryWidget = XmtBuildQueryDialog (toplevel, "quSingle", 
		squery_resource, XtNumber (squery_resource), NULL);

	memset ((void *)&single, 0, sizeof (squery_t));
	ActivateSearch (False);
	ActivateMove (False);
	sqlQuery = False;

	XmtDialogDoSync (QueryWidget, &single);
	XmtDisplayDefaultCursor (toplevel);
}

void QuerySingleCallback (Widget w, XtPointer tag, XtPointer client_data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256], queryvalue[128];
	char songa[41], songb[41], singer[41], date[11], zeit[6], zeitb[6];
	char number[11];
	char qu[7][50];
	long idate;
EXEC SQL END DECLARE SECTION;
char hv0[128], *p;
squery_t single;
int i;

	XmtDisplayBusyCursor (QueryWidget);
	XmtDialogGetDialogValues (QueryWidget, &single);
	strcpy (sql_string, "select disk.number, disk.title, disk.b_side, disk.singer, disk.pressed, disk.zeit, disk.zeitb from disk where ");
	queryvalue[0] = 0;

	for (i = 0; i < 7; i++)
	   qu[i][0] = 0;

	if (single.number[0])
	   sprintf (qu[0], "disk.number >= %s", single.number);

	if (single.songa[0])
	   sprintf (qu[1], "disk.title like '%s'", single.songa);

	if (single.songb[0])
	   sprintf (qu[2], "disk.b_side like '%s'", single.songb);

	if (single.singer[0])
	   sprintf (qu[3], "disk.singer like '%s'", single.singer);

	if (single.date[0])
	   sprintf (qu[4], "disk.date = %s", single.date);

	if (isdigit (single.zeit[0]))
	   sprintf (qu[5], "disk.zeit like '%s'", single.zeit);

	if (isdigit (single.zeit[0]))
	   sprintf (qu[6], "disk.zeitb like '%s'", single.zeitb);

	for (i = 0; i < 7; i++)
	{
	   if (qu[i][0])
	   {
	      if (queryvalue[0])
		 strcat (queryvalue, " and ");

	      strcat (queryvalue, qu[i]);
	   }
	}

	if (!strlen (queryvalue))
	{
	   XmtDisplayDefaultCursor (QueryWidget);
	   return;
	}

	/*
	 * Damit die Suche besser aussieht, sortieren wir das Suchergebnis
         * nach jenem Feld welches als erstes einen Suchbegriff enthaelt.
	 */
	strcpy (hv0, queryvalue);

	if ((p = strchr (hv0, ' ')) != NULL)
	{
	   *p = 0;
	   strcat (queryvalue, " order by ");
	   strcat (queryvalue, hv0);
	}

	strcat (sql_string, queryvalue);

	EXEC SQL prepare siid from :sql_string;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   fprintf (stderr, "platten: Error: 'prepare siid from %s'\n", sql_string);
	   DisplaySqlError (QueryWidget);
	   XmtDisplayDefaultCursor (QueryWidget);
	   return;
	}

	EXEC SQL declare plcurs scroll cursor for siid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   fprintf (stderr, "platten: Error: 'declare plcurs scroll cursor for siid'\n");
	   DisplaySqlError (QueryWidget);
	   XmtDisplayDefaultCursor (QueryWidget);
	   return;
	}

	EXEC SQL open plcurs;
	EXEC SQL fetch plcurs into :number, :songa, :songb, :singer, :idate, :zeit, :zeitb;

	if (SQLCODE != 0 && CheckSqlError ())
	{
	   if (SQLCODE == 100)
	   {
	      XmtDisplayError (QueryWidget, "iNotFound", NULL);
	      XmtDisplayDefaultCursor (QueryWidget);
	      return;
	   }

	   fprintf (stderr, "platten: Error: 'fetch plcurs into :number, :songa, :songb, :singer, :idate, :zeit, :zeitb'\n");
	   DisplaySqlError (QueryWidget);
	   XmtDisplayDefaultCursor (QueryWidget);
	   return;
	}

	sqlQuery = True;
	strcpy (single.number, number);
	strcpy (single.songa, songa);
	strcpy (single.songb, songb);
	strcpy (single.singer, singer);

	if (isdigit (zeit[0]))
	   strcpy (single.zeit, zeit);
	else
	   single.zeit[0] = 0;

	if (isdigit (zeitb[0]))
	   strcpy (single.zeitb, zeitb);
	else
	   single.zeitb[0] = 0;

	if (idate < 1000L)
	   single.date[0] = 0;
	else
	   IDateToChar (idate, &single.date[0]);

	XmtDialogSetDialogValues (QueryWidget, &single);
	ActivateSearch (False);
	ActivateMove (True);
	XmtDisplayDefaultCursor (QueryWidget);
}

void NewSingleCallback (Widget w, XtPointer tag, XtPointer client_data)
{
squery_t single;

	memset ((void *)&single, 0, sizeof (squery_t));
	XmtDialogSetDialogValues (QueryWidget, &single);
	ActivateSearch (False);
	ActivateMove (False);
	sqlQuery = False;
}

void NextSingleCallback (Widget w, XtPointer tag, XtPointer client_data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char songa[41], songb [41], singer[41], date[11], zeit[6], zeitb[6];
	char number[11];
	long idate;
EXEC SQL END DECLARE SECTION;
squery_t single;

	XmtDisplayBusyCursor (QueryWidget);
	zeit[0] = 0;
	EXEC SQL fetch plcurs into :number, :songa, :songb, :singer, :idate, :zeit, :zeitb;

	if (strncmp (SQLSTATE, "00", 2) != 0 && SQLCODE != 0)
	{
	   fprintf (stderr, "platten: Error: 'fetch plcurs into :number, :songa, :songb, :singer, :idate, :zeit'\n");
	   XmtDisplayError (QueryWidget, "iNotDirection", NULL);
	   XmtDisplayDefaultCursor (QueryWidget);
	   return;
	}

	strcpy (single.number, number);
	strcpy (single.songa, songa);
	strcpy (single.songb, songb);
	strcpy (single.singer, singer);

	if (isdigit (zeit[0]))
	   strcpy (single.zeit, zeit);
	else
	   single.zeit[0] = 0;

	if (isdigit (zeitb[0]))
	   strcpy (single.zeitb, zeitb);
	else
	   single.zeitb[0] = 0;

	if (idate < 1000L)
	   single.date[0] = 0;
	else
	   IDateToChar (idate, &single.date[0]);

	XmtDialogSetDialogValues (QueryWidget, &single);
	ActivateSearch (False);
	XmtDisplayDefaultCursor (QueryWidget);
}

void PreviousSingleCallback (Widget w, XtPointer tag, XtPointer client_data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char songa[41], songb [41], singer[41], date[11], zeit[6], zeitb[6];
	char number[11];
	long idate;
EXEC SQL END DECLARE SECTION;
squery_t single;

	XmtDisplayBusyCursor (QueryWidget);
	zeit[0] = 0;
	EXEC SQL fetch previous plcurs into :number, :songa, :songb, :singer, :idate, :zeit, :zeitb;

	if (strncmp (SQLSTATE, "00", 2) != 0 && SQLCODE != 0)
	{
	   fprintf (stderr, "platten: Error: 'fetch previous plcurs into :number, :songa, :songb, :singer, :idate, :zeit'\n");
	   XmtDisplayError (QueryWidget, "iNotDirection", NULL);
	   XmtDisplayDefaultCursor (QueryWidget);
	   return;
	}

	strcpy (single.number, number);
	strcpy (single.songa, songa);
	strcpy (single.songb, songb);
	strcpy (single.singer, singer);

	if (isdigit (zeit[0]))
	   strcpy (single.zeit, zeit);
	else
	   single.zeit[0] = 0;

	if (isdigit (zeitb[0]))
	   strcpy (single.zeitb, zeitb);
	else
	   single.zeitb[0] = 0;

	if (idate < 1000L)
	   single.date[0] = 0;
	else
	   IDateToChar (idate, &single.date[0]);

	XmtDialogSetDialogValues (QueryWidget, &single);
	ActivateSearch (False);
	XmtDisplayDefaultCursor (QueryWidget);
}

void OkaySingleCallback (Widget w, XtPointer tag, XtPointer client_data)
{
Widget shell;
XmtDialogInfo *dialog;

	shell = XmtGetShell (w);
	dialog = _XmtDialogLookupDialogInfo (shell);

	if (!dialog)
	   return;

	XmtDialogGetDialogValues (shell, dialog->base);
	dialog->return_value = True;
	dialog->blocking = False;
	ActivateSearch (True);
	ActivateMove (True);
	XtUnmanageChild (((CompositeWidget)shell)->composite.children[0]);

	if (sqlQuery)
	{
	   if (has_trans)
	      EXEC SQL commit work;

	   EXEC SQL close plcurs;
	   EXEC SQL free plcurs;
	   sqlQuery = False;
	}
}

void SingleValueChangedCallback (Widget w, XtPointer tag, XtPointer client_data)
{
int len;
String st = (String)client_data;

	if (st == NULL)
	   return;

	len = strlen (st);

	if (len > 0)
	   ActivateSearch (True);
	else
	   ActivateSearch (False);
}

void ActivateSearch (Boolean b)
{
Widget w = XmtNameToWidget (toplevel, "*quSingle*suchen");

	XtSetSensitive (w, b);
}

void ActivateMove (Boolean b)
{
Widget wNext = XmtNameToWidget (toplevel, "*quSingle*next");
Widget wPrevious = XmtNameToWidget (toplevel, "*quSingle*previous");
Widget wChange = XmtNameToWidget (toplevel, "*quSingle*change");

	XtSetSensitive (wNext, b);
	XtSetSensitive (wPrevious, b);
	XtSetSensitive (wChange, b);
}
